@@ -52,7 +52,9 @@ l = Lark(r"""
     enum_definition.2: modifier? "enum" IDENTIFIER_NAME [":" IDENTIFIER_NAME] "{" enum_block? "}"
     enum_block: (IDENTIFIER_NAME ",")* IDENTIFIER_NAME
 
-    class_definition.2: (modifier | "partial" | "sealed" | "internal")* "class" IDENTIFIER_NAME [":" type_name] "{" class_block "}"
+    class_definition.2: (modifier | "partial" | "sealed" | "internal")* "class" type_name [":" class_inhereted_classes] [class_constraints] "{" class_block "}"
+    class_inhereted_classes: (type_name ",")* type_name
+    class_constraints: "where" IDENTIFIER_NAME ":" (type_name ",")* type_name
     class_block: (method | attribute | class_definition)*
 
     attribute.2: modifier? type_name IDENTIFIER_NAME (("=" ATTRIBUTE_VALUE) | ";")
@@ -110,14 +112,16 @@ def find_namespace_names(tree: ParseTree) -> Iterable[tuple[ParseTree, str]]:
 
 def find_class_names(tree: ParseTree) -> Iterable[tuple[ParseTree, str]]:
     for class_node in find_direct_data(tree, "class_definition"):
-        name = find_direct_child_token(class_node, "IDENTIFIER_NAME")
+        # print(find_type_name(class_node))
+        # name = find_direct_child_token(class_node, "IDENTIFIER_NAME")
+        name = find_type_name(class_node)
         assert name
-        yield (class_node, name.value)
+        yield (class_node, name)
 
         class_block = find_direct_child_rule(class_node, "class_block")
         assert class_block
         for (sub_class_node, sub_class_name) in find_class_names(class_block):
-            yield (sub_class_node, name.value+"."+sub_class_name)
+            yield (sub_class_node, name+"."+sub_class_name)
 
 def find_visibility(tree: ParseTree) -> Optional[VisibilityEnum]:
     modifier = find_direct_child_rule(tree, "modifier")
